import traceback

import pandas as pd
import pymysql
from sqlalchemy import create_engine
from sqlalchemy.types import NVARCHAR, Float, Integer, DATE, DATETIME,BIGINT
import numpy as np

mysql_host = '192.168.101.102'
mysql_user = 'root'
mysql_password = 'bdp@mysql'

dbDic = {}

def getCursor(db):
    connection = pymysql.connect(host=mysql_host,
                                 port=3306,
                                 user=mysql_user,
                                 password=mysql_password,
                                 db=db,
                                 charset='utf8mb4')
    return connection, connection.cursor()


def getEngine(db):
    if dbDic.__contains__(db):
        return dbDic.get(db)
    engine = create_engine("mysql+mysqlconnector://%s:%s@%s:%s/%s?charset=utf8mb4" % (
        mysql_user, mysql_password, mysql_host,'3306', db))
    dbDic[db] = engine
    return engine


def getDfFromSql(db, sql):
    return pd.read_sql(sql, getEngine(db))

def getMaxId(db,table):
    sql = 'select max(auto_id) from %s' % (table)
    df = getDfFromSql(db,sql)
    return df.iloc[0,0]

def getDfFromTable(db, table):
    return pd.read_sql("select * from " + table, getEngine(db))


def executeSql(db, sql):
    con, cursor = getCursor(db)
    try:
        cursor.execute(sql)
        con.commit()
    except:
        con.rollback()
        traceback.print_exc()
    finally:
        cursor.close()
        con.close()


def dropTable(db, tableName):
    executeSql(db, "drop table if exists %s" % tableName)


def mapping_df_types(df):
    dtypeDict = {}
    for i, j in zip(df.columns, df.dtypes):
        if "object" in str(j):
            if i in ['stock_id', 'code']:
                dtypeDict.update({i: NVARCHAR(length=12)})
            elif i in ['date']:
                dtypeDict.update({i: NVARCHAR(length=12)})
            elif 'date' in i:
                dtypeDict.update({i: NVARCHAR(length=12)})
            else:
                dtypeDict.update({i: NVARCHAR(length=255)})
        if "float" in str(j):
            dtypeDict.update({i: Float(precision=2, asdecimal=True)})
        if "int" in str(j):
            dtypeDict.update({i: BIGINT})
    dtypeDict['stock_id'] = NVARCHAR(length=12)
    dtypeDict['code'] = NVARCHAR(length=12)
    dtypeDict['date'] = DATETIME
    dtypeDict['trade_date'] = DATE
    dtypeDict['ds'] = NVARCHAR(length=12)
    return dtypeDict


def saveDf(df, db, tableName, isAppend=True, index=None):
    engine = getEngine(db)
    assert isinstance(df, pd.DataFrame)
    df = df.replace([np.inf, -np.inf], np.NaN)
    df = df.fillna(np.NaN)
    saveIndex = True
    if index is None:
        saveIndex = False
    else:
        # df.reset_index(inplace=True)
        df = df.set_index(index)
        # df.drop(columns='index')
    mode = 'append' if isAppend else 'replace'
    df.to_sql(tableName, engine, if_exists=mode, dtype=mapping_df_types(df), index=saveIndex, index_label=index,chunksize=1000)

def getTablesWithAutoIncrement(db):
    sql = "SELECT table_name from information_schema.TABLES where table_schema = '%s' and auto_increment is null" % (db)
    df = getDfFromSql(db,sql)
    if df is None or df.empty:
        print('all table has increment')
        return []
    tables = df['TABLE_NAME'].tolist()
    return tables

def getTables(db):
    sql = "SELECT table_name from information_schema.TABLES where table_schema = '%s' and auto_increment is not null" % (db)
    df = getDfFromSql(db,sql)
    if df is None or df.empty:
        print('all table has increment')
        return []
    tables = df['TABLE_NAME'].tolist()
    return tables

def addAutoIncrementToDb(db):
    tables = getTablesWithAutoIncrement(db)
    step = 1
    for table in tables:
        print('add auto_id to %s : %d / %d' % (table,step, len(tables)) )
        sql = 'alter table %s add auto_id int not null primary key auto_increment first' % (table)
        executeSql(db,sql)
        step += 1

def quote(x):
    return "'" + x + "'" if isinstance(x, str) else str(x)


def insertIntoMysql(database, table, columns, values):
    if isinstance(columns, str):
        columns, values = [columns], [values]
    columns = ", ".join(columns)
    values = ", ".join([quote(value) for value in values])
    sql = "insert into %s (%s) values (%s)" % (table, columns, values)
    executeSql(database, sql)


def updateRowMysql(database, table, columns, values, primaryCol=None, primaryValue=None):
    setClause = []
    for col, val in zip(columns, values):
        if val is not None:
            setClause.append("%s = %s" % (col, quote(val)))
    sql = "update %s set " % table
    sql = sql + ", ".join(setClause)
    if primaryCol is not None and primaryValue is not None:
        if isinstance(primaryCol, str):
            sql = sql + " where %s = %s" % (primaryCol, quote(primaryValue))
        elif isinstance(primaryCol, list):
            con = []
            for col, val in zip(primaryCol, primaryValue):
                con.append("%s = %s" % (col, quote(val)))
            sql = sql + " where " + " and ".join(con)
    executeSql(database, sql)

if __name__ == '__main__':
    addAutoIncrementToDb('ods')



__author__ = 'shenli'

def addIndex(objectList):
    if objectList is None or len(objectList) == 0:
        return None
    for i in range(0,len(objectList)):
        objectList[i]['index'] = i
    return objectList


from impala.dbapi import connect
from impala.util import as_pandas


def getDfFromSql(db,sql):
    conn = connect(host="node1", port=21050, database=db)
    cursor = conn.cursor()
    cursor.execute(sql)
    df = as_pandas(cursor)
    print(df)
    cursor.close()
    conn.close()
    return df


#coding=utf-8

class CentrePosition:
    high = 'high'
    middle = 'middle'
    low = 'low'
    none = 'none'

class Direction:
    up = 1
    down = -1

class BarContainType:
    left = -1
    right = 1
    none = 0

class PartingType:
    up = 1
    down = -1
    none = 0

class PenType:
    up = 1
    down = -1

class DepartureType:
    up = 1
    down = -1
    none = 0

class CentreUpdateType:
    forwardBroken = 0
    backwardBroken = 1
    forwardMove = 2
    backwardMove = 3
    innerInclude = 4
    outerInclude = 5

class BuyPointsType:
    firstBuy = 'firstBuy'
    secondBuy = 'SecondBuy'
    thirdBuy = 'thirdBuy'
    secondThirdBuy = 'SecondThirdBuy'
    firstSell = 'firstSell'
    secondSell = 'secondSell'
    thirdSell = 'thirdSell'
    downDeparture = 'downDeparture'
    limitSell = 'limitSell'
    buySell = 'buySell'

    @staticmethod
    def getBuyPointSimple(tradeType):
        dic = {
            BuyPointsType.firstBuy : 1,
            BuyPointsType.secondBuy: 2,
            BuyPointsType.thirdBuy: 3,
            BuyPointsType.secondThirdBuy: 23,
            BuyPointsType.firstSell: 11,
            BuyPointsType.secondSell: 12,
            BuyPointsType.thirdSell: 13,
            BuyPointsType.downDeparture: 0,
            BuyPointsType.limitSell: 14,
            BuyPointsType.buySell : 8
        }
        return dic[tradeType]

    @staticmethod
    def getBuyPointDesc(tradeType):
        dic = {
            BuyPointsType.firstBuy : '一买',
            BuyPointsType.secondBuy: '二买',
            BuyPointsType.thirdBuy: '三买',
            BuyPointsType.secondThirdBuy: '二三买共振',
            BuyPointsType.firstSell: '一卖',
            BuyPointsType.secondSell: '二卖',
            BuyPointsType.thirdSell: '三卖',
            BuyPointsType.downDeparture: '盘整背驰',
            BuyPointsType.limitSell: '止损',
            BuyPointsType.buySell : '止盈'
        }
        return dic[tradeType]

    @staticmethod
    def isBuyPoints(tradeType):
        BuyTypeList = [BuyPointsType.firstBuy,BuyPointsType.secondBuy,BuyPointsType.thirdBuy,BuyPointsType.secondThirdBuy,BuyPointsType.downDeparture]
        if tradeType in BuyTypeList:
            return True
        return False

    @staticmethod
    def isSellPoints(tradeType):
        SellTypeList = [BuyPointsType.firstSell,BuyPointsType.secondSell,BuyPointsType.thirdSell,BuyPointsType.limitSell,BuyPointsType.buySell]
        if tradeType in SellTypeList:
            return True
        return False

class IndexType:
        partingIndex = 0
        penStartIndex = 1
        penEndIndex = 2
        segmentStartIndex = 3
        segmentEndIndex = 4
        centreStartIndex = 5
        centreEndIndex = 6
        trendStartIndex = 7
        trendEndIndex = 8
        histDataIndex = 9

class BuyPoint:
    def __init__(self):
        self.index = None
        self.value = None
        self.type = None
        self.date = None

class Parting:
    def __init__(self):
        self.partingType = None
        self.partingValue = None
        self.dateTime = None
        self.barIndex = -1
        self.leftBar = None
        self.middleBar = None
        self.rightBar = None

class Pen:
    def __init__(self):
        self.startDate = None
        self.endDate = None
        self.startIndex = None
        self.endIndex = None
        self.startValue = None
        self.endValue = None
        self.isValid = True

    def setValue(self,startDate,startValue,startIndex,endDate,endValue,endIndex,isValid=True):
        self.startDate = startDate
        self.startValue = startValue
        self.startIndex = startIndex
        self.endDate = endDate
        self.endValue = endValue
        self.endIndex = endIndex
        self.isValid = isValid

    def getPenType(self):
        if self.startValue > self.endValue:
            return PenType.down
        else:
            return PenType.up

    @staticmethod
    def createPenFromPartingList(partingList,startIndex,endIndex):
        startParting = partingList[startIndex]
        endParting = partingList[endIndex]
        assert isinstance(startParting,Parting)
        assert isinstance(endParting,Parting)
        pen = Pen()
        pen.startIndex = startParting.barIndex
        pen.endIndex = endParting.barIndex
        pen.startDate = startParting.dateTime
        pen.endDate = endParting.dateTime
        pen.startValue = startParting.partingValue
        pen.endValue = endParting.partingValue
        return pen

class Centre:
    def __init__(self):
        self.startIndex = None
        self.endIndex = None
        self.startDate = None
        self.endDate = None
        self.low = None
        self.high = None

    def setValue(self,startIndex,endIndex,startDate,endDate,low,high):
        self.startIndex = startIndex
        self.endIndex = endIndex
        self.startDate = startDate
        self.endDate = endDate
        self.low = low
        self.high = high



from ChangLunLogic.Model import *
import ChangLunLogic.CommonUtils as CommonUtils

class SegmentsCreateLogic:
    def __init__(self,penList):
        self.isBrokenBeginPoint = False
        self.penList = penList
        self.startIndex = 0
        self.startValue = 0.0
        self.hasSegment = False
        self.segmentList = []
        self.segmentType = None
        self.centre = {}
        self.centerNum = 0
        self.lastCenterPenNum = 0
        self.extremeIndex = 0
        self.extremePoint = 0.0
        self.isEnd = False
        self._createSegment()
        self.fixLastSegment()
        self.centreList = []

    def fixLastSegment(self):
        if len(self.segmentList) == 0:
            return
        lastSegment = self.segmentList[-1]
        if lastSegment['endPenIndex'] == len(self.penList):
            return
        if lastSegment['type'] == PenType.up and self.penList[-1]['endValue'] >= lastSegment['endValue'] \
            or lastSegment['type'] == PenType.down and self.penList[-1]['endValue'] <= lastSegment['endValue']:
            lastSegment['endValue'] = self.penList[-1]['endValue']
            lastSegment['endDate'] = self.penList[-1]['endDate']
            lastSegment['endOrginIndex'] = self.penList[-1]['endOrginIndex']
            lastSegment['endSimplifyIndex'] = self.penList[-1]['endSimplifyIndex']
            lastSegment['endPenIndex'] = len(self.penList)

    def _getNewCentreFromPen(self,pen):
        newCentre = {}
        newCentre['high'] = max(pen['startValue'],pen['endValue'])
        newCentre['low'] = min(pen['startValue'],pen['endValue'])
        return newCentre

    def _getNewCentreType(self,centre,newCentre):
        if newCentre['high'] <= centre['high'] and newCentre['low'] >= centre['low']:
            return CentreUpdateType.innerInclude
        if newCentre['high'] >= centre['high'] and newCentre['low'] <= centre['low']:
            return CentreUpdateType.outerInclude
        isBroken = newCentre['high'] <= centre['low'] or newCentre['low'] >= centre['high']
        if (newCentre['high'] > centre['high'] and self.segmentType == PenType.up) or (newCentre['low'] < centre['low'] and self.segmentType == PenType.down):
            if isBroken:
                return CentreUpdateType.forwardBroken
            else:
                return CentreUpdateType.forwardMove
        else:
            if isBroken:
                return CentreUpdateType.backwardBroken
            else:
                return CentreUpdateType.backwardMove

    def _createSegment(self):
        if self.penList is None:
            return
        if self.isEnd:
            return
        if len(self.penList[self.startIndex:]) < 3:
            return
        self._initData()
        if self.isBrokenBeginPoint:
            self._createSegmentFromPens(self.startIndex, self.extremeIndex)
            self.startIndex = self.extremeIndex + 1
            self._createSegment()
            return
        nextPenIndex = self.startIndex + 3
        while nextPenIndex < len(self.penList):
            nextPen = self.penList[nextPenIndex]
            newCentre = self._getNewCentreFromPen(nextPen)
            centreUpdateType = self._getNewCentreType(self.centre,newCentre)
            if centreUpdateType == CentreUpdateType.forwardBroken or centreUpdateType == CentreUpdateType.forwardMove:
                if centreUpdateType == CentreUpdateType.forwardBroken:
                    self.centerNum += 1
                    self.lastCenterPenNum = 1
                else:
                    self.lastCenterPenNum += 1
                self._updateCentre(nextPen)
                self.extremeIndex = nextPenIndex - 1
                self.extremePoint = self.penList[nextPenIndex-1]['endValue']
                self.hasSegment = True
                nextPenIndex += 2
                continue
            if centreUpdateType == CentreUpdateType.outerInclude:
                if (self.segmentType == PenType.up and nextPen['endValue'] < self.startValue) or (self.segmentType == PenType.down and nextPen['endValue'] > self.startValue):
                    self.extremeIndex = nextPenIndex - 1
                    self._createSegmentFromPens(self.startIndex,self.extremeIndex)
                    self.startIndex = self.extremeIndex + 1
                    self._createSegment()
                    return
                else:
                    self.lastCenterPenNum += 1
                    self.updateCentrePenBroken(nextPen)
                    self.extremeIndex = nextPenIndex - 1
                    self.extremePoint = self.penList[nextPenIndex - 1]['endValue']
                    nextPenIndex += 2
                    continue
            if centreUpdateType == CentreUpdateType.innerInclude:
                self.lastCenterPenNum += 1
                nextPenIndex += 2
                continue
            if centreUpdateType == CentreUpdateType.backwardMove:
                if self.hasSegment:
                    if not self.centre['isPenBroken']:
                        self._createSegmentFromPens(self.startIndex,self.extremeIndex)
                        self.startIndex = self.extremeIndex + 1
                        self._createSegment()
                        return
                    else:
                        if ( self.segmentType == PenType.up and nextPen['endValue'] < self.centre['orginLow'] ) \
                                or (self.segmentType == PenType.down and nextPen['startValue'] > self.centre['orginHigh']):
                            self._createSegmentFromPens(self.startIndex, self.extremeIndex)
                            self.startIndex = self.extremeIndex + 1
                            self._createSegment()
                            return
                        else:
                            nextPenIndex += 2
                            continue
                else:
                    if self._isPenBrokenStartPoint(nextPen):
                        self._createSegmentFromPens(self.startIndex,self.extremeIndex)
                        self.startIndex = self.extremeIndex + 1
                        self._createSegment()
                        return
                    else:
                        nextPenIndex += 2
                        continue
            if centreUpdateType == CentreUpdateType.backwardBroken:
                self._createSegmentFromPens(self.startIndex, self.extremeIndex)
                self.startIndex = self.extremeIndex + 1
                self._createSegment()
                return
        self._createSegmentFromPens(self.startIndex,self.extremeIndex)
        self.isEnd = True

    def _initData(self):
        self.isBrokenBeginPoint = False
        self.segmentType = self.penList[self.startIndex]['type']
        self.startValue = self.penList[self.startIndex]['startValue']
        if (self.segmentType == PenType.up and self.penList[self.startIndex+2]['endValue'] > self.penList[self.startIndex]['endValue'])\
                or (self.segmentType == PenType.down and self.penList[self.startIndex+2]['endValue'] < self.penList[self.startIndex]['endValue']):
            self.hasSegment = True
            self.extremeIndex = self.startIndex + 2
            self.extremePoint = self.penList[self.startIndex+2]['endValue']
        else:
            self.hasSegment = False
            self.extremeIndex = self.startIndex
            self.extremePoint = self.penList[self.startIndex]['endValue']
            if self.segmentType == PenType.up and self.penList[self.startIndex+1]['endValue'] < self.penList[self.startIndex]['startValue']:
                self.isBrokenBeginPoint = True
            if self.segmentType == PenType.down and self.penList[self.startIndex+1]['endValue'] > self.penList[self.startIndex]['startValue']:
                self.isBrokenBeginPoint = True

        self._updateCentre(self.penList[self.startIndex+1])
        self.centerNum = 1
        self.lastCenterPenNum = 1

    def _updateCenterManual(self,startIndex,high,low):
        self.centre['high'] = high
        self.centre['low'] = low
        self.centre['startIndex'] = startIndex

    def updateCentrePenBroken(self,nextPen):
        if nextPen['type'] == PenType.down:
            self.centre['high'] = nextPen['startValue']
            self.centre['startIndex'] = nextPen['index']
            self.centre['isPenBroken'] = True
            self.centre['orginLow'] = nextPen['endValue']
            self.centre['orginHigh'] = nextPen['startValue']
        else:
            self.centre['low'] = nextPen['startValue']
            self.centre['startIndex'] = nextPen['index']
            self.centre['isPenBroken'] = True
            self.centre['orginLow'] = nextPen['startValue']
            self.centre['orginHigh'] = nextPen['endValue']

    def _updateCentre(self,nextPen):
        if nextPen['type'] == PenType.down:
            self.centre['high'] = nextPen['startValue']
            self.centre['low'] = nextPen['endValue']
            self.centre['startIndex'] = nextPen['index']
            self.centre['isPenBroken'] = False
            self.centre['orginLow'] = nextPen['endValue']
            self.centre['orginHigh'] = nextPen['startValue']
        else:
            self.centre['high'] = nextPen['endValue']
            self.centre['low'] = nextPen['startValue']
            self.centre['startIndex'] = nextPen['index']
            self.centre['isPenBroken'] = False
            self.centre['orginLow'] = nextPen['startValue']
            self.centre['orginHigh'] = nextPen['endValue']

    def _updateExtrePoints(self,nextPen):
        if nextPen['type'] == PenType.down:
            if nextPen['startValue'] > self.extremePoint:
                self.extremePoint = nextPen['startValue']
                self.extremeIndex = nextPen['index']
            if nextPen['endValue'] < self.revesreExtrePoint:
                self.revesreExtrePoint = nextPen['endValue']
                self.revesreExtreIndex = nextPen['index']
        else:
            if nextPen['startValue'] < self.extremePoint:
                self.extremePoint = nextPen['startValue']
                self.extremeIndex = nextPen['index']
            if nextPen['endValue'] > self.revesreExtrePoint:
                self.revesreExtrePoint = nextPen['endValue']
                self.revesreExtreIndex = nextPen['index']

    def _isPenBrokenStartPoint(self,nextPen):
        startValue = self.penList[self.startIndex]['startValue']
        if self.segmentType == PenType.down and nextPen['endValue'] > startValue:
            return True
        if self.segmentType == PenType.up and nextPen['endValue'] < startValue:
            return True
        return False

    def _isCentreAndPenOverlap(self,center,nextPen):
        minPen = min(nextPen['startValue'],nextPen['endValue'])
        maxPen = max(nextPen['startValue'],nextPen['endValue'])
        if center['low'] >= maxPen or center['high'] <= minPen:
            return False
        return True

    def _isNewCentreForward(self,nextPen):
        if nextPen['type'] == PenType.up and nextPen['endValue'] < self.centre['low']:
            return True
        if nextPen['type'] == PenType.down and nextPen['endValue'] > self.centre['high']:
            return True
        return False

    def _createForwardCentreSegments(self,nextPenType):
        assert self.centrePenCount > 3
        extremePenIndex = self.extremeIndex
        if self.extremeIndex > self.revesreExtreIndex:
            if nextPenType == PenType.down:
                extremePen = max(self.penList[self.startIndex + 1:self.revesreExtreIndex + 1],key = lambda x: x['startValue'])
            else:
                extremePen = min(self.penList[self.startIndex + 1:self.revesreExtreIndex + 1],key = lambda x: x['startValue'])
            extremePenIndex = extremePen['index']
        self._createSegmentFromPens(self.startIndex,extremePenIndex - 1)
        self._createSegmentFromPens(extremePenIndex,self.revesreExtreIndex)
        self._setNextStart(self.revesreExtreIndex+1)

    def _resetCentrePenCount(self):
        self.centrePenCount = 0

    def _setNextStart(self,endIndex):
        self.startIndex = endIndex

    def _createSegmentFromPens(self,startIndex, endIndex):
        segment = {}
        segment['startValue'] = self.penList[startIndex]['startValue']
        segment['startDate'] = self.penList[startIndex]['startDate']
        segment['startOrginIndex'] = self.penList[startIndex]['startOrginIndex']
        segment['startSimplifyIndex'] = self.penList[startIndex]['startSimplifyIndex']
        segment['startPenIndex'] = startIndex
        segment['endValue'] = self.penList[endIndex]['endValue']
        segment['endDate'] = self.penList[endIndex]['endDate']
        segment['endOrginIndex'] = self.penList[endIndex]['endOrginIndex']
        segment['endSimplifyIndex'] = self.penList[endIndex]['endSimplifyIndex']
        segment['endPenIndex'] = endIndex
        segment['type'] = self.penList[startIndex]['type']
        if endIndex - startIndex < 2:
            segment['isValid'] = False
        else:
            segment['isValid'] = True
        segment['centerNum'] = self.centerNum
        segment['lastCenterPenNum'] = self.lastCenterPenNum
        self.segmentList.append(segment)

def createSegmentList(penList):
    segmentLogic = SegmentsCreateLogic(penList)
    segmentList = CommonUtils.addIndex(segmentLogic.segmentList)
    return segmentList







from pyecharts import options as opts
from pyecharts.charts import Kline, Line, Scatter

import UtilTools.DateUtils as DateUtils
from ChangLunLogic.Model import *


class ChanLunPlot():
    def __init__(self, chanLunRecursieLogic):
        self.features = None
        self.chunLunModel = chanLunRecursieLogic

    def setFeatures(self,features):
        self.features = features

    def plot(self, basePath = 'D:\\stock\\'):
        simplifyBars = self.getSimplifyLines()
        penLine = self.getPenLines()
        if penLine is not None:
            simplifyBars.overlap(penLine)
        segmentLines = self.getSegmentLines()
        if segmentLines is not None:
            for line in segmentLines:
                simplifyBars.overlap(line)
        if self.features is not None:
            histBuyPoints = self.getHistBuyPoints()
            simplifyBars.overlap(histBuyPoints)
        simplifyBars.render('%s\\%s.html' % (basePath,self.chunLunModel.stockId))

    def save(self, path=None, endDate=None):
        if path is None:
            if endDate is None:
                endDate = DateUtils.getTodayStr()
            basePath = 'D:\\xampp\\htdocs\\model\\%s' % (endDate)
            if self.chunLunModel.buyPoint is not None:
                simpleBuyPoint = BuyPointsType.getBuyPointSimple(self.chunLunModel.buyPoint.type)
            else:
                simpleBuyPoint = 9
            path = '%s\\%s_%s%s.html' % (
            basePath, str(simpleBuyPoint), self.chunLunModel.stockId, self.chunLunModel.name)
        print(path)
        self.overlap.render(path)

    def plotHistBuyPoint(self, buyPoints):
        if buyPoints is None or len(buyPoints) == 0:
            return
        buyPointDic = {}
        for x in buyPoints:
            type = x['type']
            date = x['date']
            value = x['value']
            if buyPointDic.__contains__(type):
                buyPointDic[type][0].append(date)
                buyPointDic[type][1].append(value)
            else:
                buyPointDic[type] = ([], [])
        for type in buyPointDic.keys():
            pointPlot = Scatter()
            pointPlot.add(type, buyPointDic[type][0], buyPointDic[type][1])
            self.overlap.add(pointPlot)

    def getSimplifyLines(self):
        olchList = []
        datetimeList = []
        for bar in self.chunLunModel.histData:
            barDate = DateUtils.convertTimestampToStr(bar['date'])
            olch = [bar['open'], bar['close'], bar['low'], bar['high']]
            datetimeList.append(barDate)
            olchList.append(olch)
        kline = (
            Kline(
                init_opts=opts.InitOpts(width="100%",height="600px")
            ).add_xaxis(xaxis_data=datetimeList)
                .add_yaxis(series_name='', y_axis=olchList, itemstyle_opts=opts.ItemStyleOpts(
                color="#ef232a",
                color0="#14b143",
                border_color="#ef232a",
                border_color0="#14b143",
            )).set_global_opts(
                xaxis_opts=opts.AxisOpts(is_scale=True),
                yaxis_opts=opts.AxisOpts(
                    is_scale=True,
                    splitarea_opts=opts.SplitAreaOpts(
                        is_show=True, areastyle_opts=opts.AreaStyleOpts(opacity=1)
                    ),
                ),
                datazoom_opts=[opts.DataZoomOpts(range_start=80,range_end=100)],
                title_opts=opts.TitleOpts(title=self.chunLunModel.stockId),

            )
        )

        centre = self.getPenCentreMarkItemAreas()
        if centre is not None and len(centre) > 0:
            kline.set_series_opts(
                markarea_opts=opts.MarkAreaOpts(data=centre)
            )

        return kline

    def getPenLines(self):
        penList = self.chunLunModel.penList
        if penList is None or len(penList) == 0:
            return None

        x = []
        y = []
        for pen in penList:
            startDate = DateUtils.convertTimestampToStr(pen['startDate'])
            x.append(startDate)
            y.append(pen['startValue'])
        endDate = DateUtils.convertTimestampToStr(penList[-1]['endDate'])
        x.append(endDate)
        y.append(penList[-1]['endValue'])
        penLine = (
            Line()
                .add_xaxis(xaxis_data=x)
                .add_yaxis(series_name='pen', y_axis=y)
        )
        return penLine

    def getSegmentLines(self):
        segmentDic = self.chunLunModel.segmentDic
        if segmentDic is None or len(segmentDic) == 0:
            return None
        segmentLines = []

        for freq in segmentDic.keys():
            if segmentDic[freq] is None or freq == '1':
                continue
            title = 'seg_%s' % (freq)
            x = []
            y = []
            for pen in segmentDic[freq]:
                startDate = DateUtils.convertTimestampToStr(pen['startDate'])
                x.append(startDate)
                y.append(pen['startValue'])
            endDate = DateUtils.convertTimestampToStr(segmentDic[freq][-1]['endDate'])
            x.append(endDate)
            y.append(segmentDic[freq][-1]['endValue'])
            segmentLine = (
                Line()
                    .add_xaxis(xaxis_data=x)
                    .add_yaxis(series_name=title, y_axis=y)
            )
            segmentLines.append(segmentLine)
        return segmentLines

    def getCentreMarkItemAreas(self,freq):
        centreList = self.chunLunModel.centreDic.get(freq)
        if centreList is None or len(centreList) == 0:
            return None
        area = []
        for centre in centreList:
            x = (centre['startDate'], centre['endDate'])
            y = (centre['low'],centre['high'])
            area.append( opts.MarkAreaItem(x=x,y=y) )
        return area

    def getPenCentreMarkItemAreas(self):
        if self.chunLunModel.penCentre is None or len(self.chunLunModel.penCentre) == 0:
            return None
        area = []
        for centre in self.chunLunModel.penCentre:
            x = (centre['startDate'], centre['endDate'])
            y = (centre['low'],centre['high'])
            area.append( opts.MarkAreaItem(x=x,y=y) )
        return area

    def getHistBuyPoints(self):
        x = [x['buy_date'] for x in self.features]
        y = [x['buy_value'] for x in self.features]
        scatter = (
            Scatter().add_xaxis(xaxis_data=x)
            .add_yaxis(series_name='历史买点',y_axis=y)
        )
        return scatter



from ChangLunLogic.ChangLun import ChangLun

def getHistTradeFeatures(df,windows=250):
    featureList = []
    if len(df) < 250:
        return []
    for i in range(len(df)-windows+1):
        tempDf = df.iloc[i:i+windows]
        model = ChangLun(tempDf,'D','M')
        isRun = model.run(True)
        if not isRun:
            continue
        isBuy = model.isPenBuyPoint()
        if not isBuy:
            continue
        feature = model.genFeature()
        if len(feature) == 0:
            continue
        featureList.append(feature)
    featureList = markFeautures(featureList,df)
    return featureList

def markFeautures( featureList, df ):
    if featureList is None or len(featureList) == 0:
        return []
    model = ChangLun(df,'D','M')
    model.run()
    lastPen = model.penList[-1]
    penSet = set([x['endDate'] for x in model.penList[0:-1]])
    for feature in featureList:
        penEndDate = feature['pen_end_date']
        isTruePen = penSet.__contains__(penEndDate)
        feature['label_is_pen_true'] = isTruePen
        feature['label_is_last_pen'] = (penEndDate == lastPen['endDate'])
    return featureList



__author__ = 'shenli'
import ChangLunLogic.CommonUtils as CommonUtils

def createCentreList(segmentList,penList):
    if segmentList is None:
        return None
    centreList = []
    for i in range(0,len(segmentList)):
        startIndex,endIndex = _getSegmentStartEndIndex(segmentList,i,penList)
        tempList = _createCentreListFromPens(penList,startIndex,endIndex,i)
        centreList.extend(tempList)
    centreList = CommonUtils.addIndex(centreList)
    return centreList

def _createCentreListFromPens(penList,startIndex,endIndex,segmentIndex):
    if startIndex == endIndex:
        return []
    centreStart = startIndex + 1
    centreEnd = centreStart + 2
    centreList = []
    while centreEnd < endIndex:
        if _isPenOverlap(penList[centreStart],penList[centreEnd]):
            centreEnd += 2
            continue
        centre = _createCentre(penList,centreStart,centreEnd-2,segmentIndex)
        centreList.append(centre)
        centreStart = centreEnd
        centreEnd = centreStart + 2
    if centreEnd >= endIndex and centreEnd - 2 >= centreStart:
        centre = _createCentre(penList,centreStart,centreEnd-2,segmentIndex)
        centreList.append(centre)
    return centreList

def _createCentre(penList,centreStart,centreEnd,segmentIndex):
    centre = {}
    centre['startDate'] = penList[centreStart]['startDate']
    centre['endDate'] = penList[centreEnd]['endDate']
    centre['startOrginIndex'] = penList[centreStart]['startOrginIndex']
    centre['endOrginIndex'] = penList[centreEnd]['endOrginIndex']
    centre['startPenIndex'] = centreStart
    centre['endPenIndex'] = centreEnd
    centre['segmentIndex'] = segmentIndex
    centre['low'] = min(penList[centreStart]['startValue'],penList[centreStart]['endValue'])
    centre['high'] = max(penList[centreStart]['startValue'],penList[centreStart]['endValue'])
    maxStart = max(penList[centreStart:centreEnd+1],key = lambda x:x['startValue'])['startValue']
    minStart = min(penList[centreStart:centreEnd+1],key = lambda x:x['startValue'])['startValue']
    maxEnd = max(penList[centreStart:centreEnd+1],key = lambda x:x['endValue'])['endValue']
    minEnd = min(penList[centreStart:centreEnd+1],key = lambda x:x['endValue'])['endValue']
    centre['extremeHigh'] = max(maxStart,maxEnd)
    centre['extremeLow'] = min(minStart,minEnd)
    return centre


def _isPenOverlap(pen1,pen2):
    min1 = min(pen1['startValue'],pen1['endValue'])
    max1 = max(pen1['startValue'],pen1['endValue'])
    min2 = min(pen2['startValue'],pen2['endValue'])
    max2 = max(pen2['startValue'],pen2['endValue'])
    if min1 > max2 or max1 < min2:
        return False
    return True

def _getSegmentStartEndIndex(segmentList,index,penList):
    startIndex = segmentList[index]['startPenIndex']
    endIndex = segmentList[index]['endPenIndex']
    #if index == len(segmentList) - 1:
        #endIndex = len(penList) - 1
    return (startIndex,endIndex)



from ChangLunLogic.Model import *

def simplifyHistData(histData):
    if histData is None or len(histData) <= 2:
        return None
    simplifyData = []
    simplifyData.append(_createSimplifyDataFromHistData(histData,0))
    simplifyData[-1]['index'] = len(simplifyData) - 1
    for i in range(1,len(histData)):
        simplifyNextBar(simplifyData,histData,i)
    return simplifyData

def simplifyNextBar(simplifyData,histData,i):
    isBarContain = _getBarContainType(simplifyData[-1], histData[i])
    if isBarContain == BarContainType.none:
        newData = _createSimplifyDataFromHistData(histData, i)
        simplifyData.append(newData)
        simplifyData[-1]['index'] = len(simplifyData) - 1
        return

    direction = _getDirection(simplifyData)
    if isBarContain == BarContainType.left:
        if direction == Direction.down:
            simplifyData[-1]['high'] = histData[i]['high']
        else:
            simplifyData[-1]['low'] = histData[i]['low']
    else:
        if direction == Direction.down:
            simplifyData[-1]['low'] = histData[i]['low']
            simplifyData[-1]['orginIndex'] = i
            simplifyData[-1]['date'] = histData[i]['date']
        else:
            simplifyData[-1]['high'] = histData[i]['high']
            simplifyData[-1]['orginIndex'] = i
            simplifyData[-1]['date'] = histData[i]['date']

def _getDirection(simplifyData):
    if len(simplifyData) < 2:
        return Direction.down
    assert _getBarContainType(simplifyData[-2],simplifyData[-1]) == BarContainType.none
    if simplifyData[-1]['high'] > simplifyData[-2]['high']:
        return Direction.up
    return Direction.down

def _createSimplifyDataFromHistData(histData,index):
    simplifyBar = {}
    simplifyBar['high'] = histData[index]['high']
    simplifyBar['low'] = histData[index]['low']
    simplifyBar['date'] = histData[index]['date']
    simplifyBar['orginIndex'] = index
    return simplifyBar

def _getBarContainType(barLeft,barRigh):
    if barLeft['high'] >= barRigh['high'] and barLeft['low'] <= barRigh['low']:
        return BarContainType.left
    if barLeft['high'] <= barRigh['high'] and barLeft['low'] >= barRigh['low']:
        return BarContainType.right
    return BarContainType.none




from datetime import datetime
from datetime import timedelta
import pandas as pd
import UtilTools.FrequencyUtils as FrequencyUtils
from dateutil.relativedelta import relativedelta

def convertDateStrToDatetime(dateStr):
    if isinstance(dateStr,pd.Timestamp):
        return dateStr.to_datetime()
    if isinstance(dateStr,datetime):
        return dateStr.replace(microsecond=0)
    if len(dateStr) == len('2016-01-01'):
        return datetime.strptime(dateStr,'%Y-%m-%d')
    elif len(dateStr) == len('2016-01-01 00:00:00'):
        return datetime.strptime(dateStr,'%Y-%m-%d %H:%M:%S')
    elif len(dateStr) == len('2016-01-01 00:00'):
        return datetime.strptime(dateStr,'%Y-%m-%d %H:%M')
    elif len(dateStr) == len('20160101'):
        return datetime.strptime(dateStr,'%Y%m%d')
    elif len(dateStr) == len('20160101 00:00:00'):
        return datetime.strptime(dateStr,'%Y%m%d %H:%M:%S')
    elif len(dateStr) == len('2018-08-20T10:00:00Z'):
        return datetime.strptime(dateStr,'%Y-%m-%dT%H:%M:%SZ')
    else:
        raise Exception("Date Format is invalid")
    return dateStr

def convertDateToTheTradeStart(dateStr):
    if isinstance(dateStr,datetime):
        date = dateStr
    else:
        date = convertDateStrToDatetime(dateStr)
    date = date.replace(hour=9, minute=0, second=0)
    return date

def convertDateToTheTradeEnd(dateStr):
    if isinstance(dateStr,datetime):
        date = dateStr
    else:
        date = convertDateStrToDatetime(dateStr)
    date = date.replace(hour=23,minute=30,second=0)
    return date

def convertDateToStr(date):
    if isinstance(date, datetime):
        return date.strftime('%Y-%m-%d')
    return date

def convertTimestampToStr(timestmap):
    if isinstance(timestmap,pd.Timestamp):
        return timestmap.strftime('%Y-%m-%d %H:%M:%S')
    if isinstance(timestmap,datetime):
        return timestmap.strftime('%Y-%m-%d %H:%M:%S')
    return timestmap

def getTodayStr():
    return datetime.now().strftime('%Y-%m-%d')

def getLastOneYearDate():
    today = getTodayStr()
    oneYearBefore = getNMonthBeforDateStr(today,12)
    return (oneYearBefore,today)

def getNMonthBeforDateStr(monthNum):
    today = getToday()
    delta = relativedelta(months=monthNum)
    before =  today - delta
    return before.strftime('%Y-%m-%d')

def getNMonthBeforDateStr(dateStr,monthNum):
    today = datetime.strptime(dateStr,'%Y-%m-%d')
    delta = relativedelta(months=monthNum)
    before =  today - delta
    return before.strftime('%Y-%m-%d')

def getMacdStartDate(dateStr,freq):
    orginDate = convertDateStrToDatetime(dateStr)
    count = 33
    timeDiff = timedelta(days=1)
    if freq == FrequencyUtils.TradeType.D:
        timeDiff = timedelta(days=1)
    elif freq == FrequencyUtils.TradeType.W:
        timeDiff = timedelta(days=7)
    elif freq == FrequencyUtils.TradeType.M:
        timeDiff = timedelta(days=31)
    elif freq == FrequencyUtils.TradeType.Sixty:
        count = count / 4
    elif freq == FrequencyUtils.TradeType.Thirty:
        count = count / 8
    elif freq == FrequencyUtils.TradeType.Fifteen:
        count = count / 16
    elif freq == FrequencyUtils.TradeType.Five or freq == FrequencyUtils.TradeType.One:
        count = 1
    macdDate = orginDate - count * timeDiff * 2
    return macdDate

def getDayDiff(date1,date2):
    date1 = convertDateStrToDatetime(date1)
    date2 = convertDateStrToDatetime(date2)
    return (date2 - date1).days

def convertDateToPaht(date):
    return date.strftime('%Y_%m_%d')

def getNow():
    return datetime.now()

def getToday():
    now = datetime.now()
    today = datetime(now.year,now.month,now.day)
    return today

def getLastTradeDayStr():
    today = getToday()
    timeDiff = timedelta(days=1)
    if datetime.now().hour < 9:
        today = today - timeDiff
    if today.isoweekday() <= 5:
        return today.strftime('%Y-%m-%d')
    else:
        yesterday = today - timeDiff
        while yesterday.isoweekday() > 5:
            yesterday = yesterday - timeDiff
        return yesterday.strftime('%Y-%m-%d')

def getNMonthBeforDate(monthNum):
    today = getToday()
    delta = relativedelta(months=monthNum)
    return  today - delta

def isDateExceedNMonth(startDate,monthNum):
    today = getToday()
    delta = relativedelta(months=monthNum)
    startDate = convertDateStrToDatetime(startDate)
    if startDate >= today - delta:
        return False
    return True

def convertTimeToDate(date):
    assert isinstance(date,datetime)
    dayDate = datetime(date.year,date.month,date.day)
    return dayDate







import os
import traceback

backTestDirTemplate = 'D:\\xampp\\htdocs\\backtest\\%s'
backTestPathTemplate = 'D:\\xampp\\htdocs\\backtest\\%s\\%s.html'
baseDirTemplate = 'D:\\xampp\\htdocs\\model\\%s'

def createDir(todayStr):
    basePath = baseDirTemplate % (todayStr)
    if os.path.exists(basePath):
        return
    try:
        os.mkdir(basePath)
    except Exception as e:
        print(e)
        traceback.print_exc()

def createBackTestDir(modelName):
    basePath = backTestDirTemplate % (modelName)
    if os.path.exists(basePath):
        return
    try:
        os.mkdir(basePath)
    except Exception as e:
        print(e)
        traceback.print_exc()

def getBackTestPath(modelName,stockId):
    return backTestPathTemplate % (modelName,stockId)




from UtilTools.impalaUtils import getDfFromSql
from ChangLunLogic.ChangLun import ChangLun
from Plot.ChanLunPlot import ChanLunPlot

if __name__ == '__main__':
    stockId = '000792'
    sql = "SELECT stock_id, trade_date as `date`, open,close,low,high from dwd_ts_stock_daily where stock_id = '%s' order by trade_date asc" % (stockId)
    df = getDfFromSql('dwd',sql)
    #df = df[df['date'] <= '20211115']
    model = ChangLun(df,'D','M')
    model.run()
    model.isPenBuyPoint()
    plot = ChanLunPlot(model)
    plot.plot()
    featureDf = model.genFeature()
    print(featureDf)


from ChangLunLogic.Model import *
import UtilTools.DateUtils as DateUtils
import UtilTools.FrequencyUtils as FreqUtils

def getThirtyLevelBuyPoint(segmentDic,endDate,rootFreq = '30'):
    superFreq = FreqUtils.getSuperFreq(rootFreq)
    subFreq = FreqUtils.getSubFreq(rootFreq)
    subSegment = segmentDic[subFreq]
    childFreq = FreqUtils.getSubFreq(subFreq)
    childSegment = None
    if childFreq is not None:
        childSegment = segmentDic[childFreq]
    superSegment = segmentDic[superFreq]
    rootSegment = segmentDic[rootFreq]
    if rootSegment is None or rootSegment[-1]['type'] == PenType.up:
        return None

    isLastRootSegmentValid = isLastSegmentValid(rootSegment[-1],subSegment)
    lastSubSegment = subSegment[rootSegment[-1]['endPenIndex'] - 1]
    isLastSubSegmentValid = isLastSegmentValid(lastSubSegment,childSegment)
    if childFreq is None and not isLastRootSegmentValid:
        return None
    if not isLastRootSegmentValid and not isLastSubSegmentValid:
        return None
    lastSegmentEndDateStr = DateUtils.convertTimestampToStr(segmentDic[rootFreq][-1]['endDate'])
    if lastSegmentEndDateStr[0:10] != endDate[0:10]:
        return None
    if superSegment is None or len(superSegment) == 0:
        return createBuyPointFromSegment(rootSegment[-1],BuyPointsType.downDeparture,rootFreq)
    if superSegment[-1]['type'] == PenType.down:
        if isSegmentValid(superSegment[-1]):
            if superSegment[-1]['endDate'] == rootSegment[-1]['endDate']:
                return createBuyPointFromSegment(rootSegment[-1],BuyPointsType.firstBuy,rootFreq)
            else:
                if len(rootSegment) - superSegment[-1]['endPenIndex'] == 3:
                    return createBuyPointFromSegment(rootSegment[-1],BuyPointsType.secondBuy,rootFreq)
                else:
                    startPenIndex = superSegment[-1]['endPenIndex'] + 1
                    if isThirdBuy(rootSegment[startPenIndex:]):
                        return createBuyPointFromSegment(rootSegment[-1],BuyPointsType.thirdBuy,rootFreq)
                    else:
                        return createBuyPointFromSegment(rootSegment[-1],BuyPointsType.downDeparture,rootFreq)
        else:
            startPenIndex = superSegment[-1]['endPenIndex'] + 1
            if len(rootSegment[startPenIndex:]) > 3 and isThirdBuy(rootSegment[startPenIndex:]):
                return createBuyPointFromSegment(rootSegment[-1],BuyPointsType.thirdBuy,rootFreq)
            else:
                return createBuyPointFromSegment(rootSegment[-1],BuyPointsType.downDeparture,rootFreq)
    else:
        startPenIndex = superSegment[-1]['startPenIndex']
        if isThirdBuy(rootSegment[startPenIndex:]):
            return createBuyPointFromSegment(rootSegment[-1],BuyPointsType.thirdBuy,rootFreq)
        else:
            return createBuyPointFromSegment(rootSegment[-1],BuyPointsType.downDeparture,rootFreq)
    return None

def isLastSegmentValid(segment,subSegment):
    if subSegment is None:
        return False
    startIndex = segment['startPenIndex']
    endIndex = segment['endPenIndex']
    if endIndex - startIndex < 2:
        return False
    nextPenIndex = startIndex + 1
    center = getCenterFromPen(subSegment[nextPenIndex])
    nextPenIndex += 2
    while nextPenIndex < endIndex:
        newCenter = getCenterFromPen(subSegment[nextPenIndex])
        if newCenter['high'] > segment['startValue'] or newCenter['low'] < segment['endValue']:
            return False
        centerType = getNewCentreType(center,newCenter)
        if centerType in [CentreUpdateType.outerInclude, CentreUpdateType.forwardMove, CentreUpdateType.backwardMove,CentreUpdateType.innerInclude]:
            return True
        elif centerType == CentreUpdateType.backwardBroken:
            return False
        else:
            center = newCenter
            nextPenIndex += 2
    return False

def isThirdBuy(segmentList):
    if len(segmentList) < 4:
        return False
    nextSegmentIndex = 1
    center = getCenterFromPen(segmentList[nextSegmentIndex])
    nextSegmentIndex += 2
    while nextSegmentIndex < len(segmentList) - 1:
        newCenter = getCenterFromPen(segmentList[nextSegmentIndex])
        centerType = getNewCentreType(center,newCenter)
        if centerType in [CentreUpdateType.outerInclude,CentreUpdateType.forwardMove,CentreUpdateType.forwardBroken]:
            center = newCenter
        nextSegmentIndex += 2
    if segmentList[-1]['endValue'] > center['high']:
        return True
    return False

def getCenterFromPen(nextPen):
    center = {}
    if nextPen['type'] == PenType.down:
        center['high'] = nextPen['startValue']
        center['low'] = nextPen['endValue']
    else:
        center['high'] = nextPen['endValue']
        center['low'] = nextPen['startValue']
    return center


def getNewCentreType(centre, newCentre):
    if newCentre['high'] <= centre['high'] and newCentre['low'] >= centre['low']:
        return CentreUpdateType.innerInclude
    if newCentre['high'] >= centre['high'] and newCentre['low'] <= centre['low']:
        return CentreUpdateType.outerInclude
    isBroken = newCentre['high'] <= centre['low'] or newCentre['low'] >= centre['high']
    if newCentre['high'] > centre['high']:
        if isBroken:
            return CentreUpdateType.forwardBroken
        else:
            return CentreUpdateType.forwardMove
    else:
        if isBroken:
            return CentreUpdateType.backwardBroken
        else:
            return CentreUpdateType.backwardMove


def createBuyPointFromSegment(segment,buyType,level):
    buyPoint = BuyPoint()
    buyPoint.date = segment['endDate']
    buyPoint.index = segment['endOrginIndex']
    buyPoint.value = segment['endValue']
    buyPoint.type = buyType
    buyPoint.level = level
    return buyPoint

def isSegmentValid(segment):
    return segment['isValid']




from ChangLunLogic.Model import *
import ChangLunLogic.CommonUtils as CommonUtils

def createPenList(partingList):
    if partingList is None:
        return None
    startIndex = _getStartIndex(partingList)
    lastIndex1 = startIndex
    lastIndex2 = None
    penList = []
    for i in range(startIndex + 1, len(partingList)):
        if _isPenExist(partingList[lastIndex1],partingList[i]):
            pen = _createPen(lastIndex1,i,partingList)
            penList.append(pen)
            lastIndex2 = lastIndex1
            lastIndex1 = i
            continue
        if len(penList) == 0:
            continue
        if _isSameTypeAndMoreHigherOrLower(partingList[lastIndex1],partingList[i]):
            pen = _createPen(lastIndex2,i,partingList)
            penList.pop(-1)
            penList.append(pen)
            lastIndex1 = i
            continue
        '''
        if len(penList) > 1 and _isSameTypeAndMoreHigherOrLower(partingList[lastIndex2],partingList[i]):
            penList.pop(-1)
            tempIndex = penList[-1]['startPartingIndex']
            pen = _createPen(tempIndex,i,partingList)
            penList.pop(-1)
            penList.append(pen)
            lastIndex2 = tempIndex
            lastIndex1 = i
        '''
    penList = CommonUtils.addIndex(penList)
    return penList

def _isSameTypeAndMoreHigherOrLower(parting1,parting2):
    if parting1['type'] != parting2['type']:
        return False
    if parting1['type'] == PartingType.up and parting2['value'] > parting1['value']:
            return True
    if parting1['type'] == PartingType.down and parting2['value'] < parting1['value']:
            return True
    return False

def _createPen(index1,index2,partingList):
    parting1 = partingList[index1]
    parting2 = partingList[index2]
    type = PenType.up
    if parting1['type'] == PartingType.up:
        type = PenType.down
    pen = {}
    pen['type'] = type
    pen['startDate'] = parting1['date']
    pen['endDate'] = parting2['date']
    pen['startValue'] = parting1['value']
    pen['endValue'] = parting2['value']
    pen['startSimplifyIndex'] = parting1['simplifyIndex']
    pen['endSimplifyIndex'] = parting2['simplifyIndex']
    pen['startOrginIndex'] = parting1['orginIndex']
    pen['endOrginIndex'] = parting2['orginIndex']
    pen['startPartingIndex'] = index1
    pen['endPartingIndex'] = index2
    return pen

def _isPenExist(parting1,parting2):
    if parting1['type'] == parting2['type']:
        return False
    type = getPartingCrossType(parting1,parting2)
    if type == 'leftContain' or type == 'rightContain':
        if parting2['simplifyIndex'] - parting1['simplifyIndex'] < 4:
            return False
        if parting1['type'] == PartingType.down and parting2['midLow'] > parting1['rightHigh']:
            return True
        if parting1['type'] == PartingType.up and parting2['midHigh'] < parting1['rightLow']:
            return True
    if type == 'upBroken' and parting1['type'] == PartingType.down and parting2['simplifyIndex'] - parting1['simplifyIndex'] >= 3:
        return True
    if type == 'downBroken' and parting1['type'] == PartingType.up and parting2['simplifyIndex'] - parting1['simplifyIndex'] >= 3:
        return True
    if type == 'upCross' and parting1['type'] == PartingType.down and parting2['simplifyIndex'] - parting1['simplifyIndex'] >= 4:
        return True
    if type == 'downCross' and parting1['type'] == PartingType.up and parting2['simplifyIndex'] - parting1['simplifyIndex'] >= 4:
        return True
    return False

def getPartingMinMax(parting):
    minValue = min(parting['leftLow'],parting['midLow'],parting['rightLow'])
    maxValue = max(parting['leftHigh'],parting['midHigh'],parting['rightHigh'])
    return (minValue,maxValue)

def getPartingCrossType(parting1,parting2):
    min1,max1 = getPartingMinMax(parting1)
    min2,max2 = getPartingMinMax(parting2)
    if min1 <= min2 and max1 >= max2 :
        return 'leftContain'
    if min2 <= min1 and max2 >= max1 :
        return 'rightContain'
    if min2 > max1:
        return 'upBroken'
    if max2 < min1:
        return 'downBroken'
    if max2 > max1:
        return 'upCross'
    if min2 < min1:
        return 'downCross'

def _getStartIndex(partingList):
    if partingList is None or len(partingList) == 0:
        return 0
    maxValue = 0
    maxIndex = 0
    minValue = 9999
    minIndex = 0
    for i in range(0,len(partingList)):
        partingType = partingList[i]['type']
        if partingType == PartingType.up and partingList[i]['value'] > maxValue:
            maxValue = partingList[i]['value']
            maxIndex = i
        if partingType == PartingType.down and partingList[i]['value'] < minValue:
            minValue = partingList[i]['value']
            minIndex = i
    startIndex = min(minIndex,maxIndex)
    startParting = partingList[startIndex]
    if startParting['orginIndex'] >= 250 and startIndex > 0:
        return _getStartIndex(partingList[0:startIndex])
    else:
        return startIndex




class TradeType:
    M = 'M'
    D = 'D'
    W = 'W'
    One = '1'
    Five = '5'
    Fifteen = '15'
    Thirty = '30'
    Sixty = '60'

FrequencyDic = {
    'M': 24*60*60*31,
    'W': 24*60*60*7,
    'D': 24*60*60,
    '60': 60*60,
    '30': 30*60,
    '15': 15*60,
    '5': 5*60,
    '1': 60
}

FrequencyMinDic = {
    '60': 60,
    '30': 30,
    '15': 15,
    '5': 5,
    '1': 1
}

FrequencyDescDic = {
    'M': '月线级别',
    'W': '周线级别',
    'D': '日线级别',
    '60': '60分钟级别',
    '30': '30分钟级别',
    '15': '15分钟级别',
    '5': '5分钟级别',
    '1': '1分钟级别'
}

SubFreqDic = {
    'M': 'W',
    'W': 'D',
    'D': '30',
    '60': '5',
    '30': '5',
    '15': '5',
    '5': '1',
    '1': None
}

SuperFreqDic = {
    'M': None,
    'W': 'M',
    'D': 'W',
    '60': 'D',
    '30': 'D',
    '15':'60',
    '5' : '30',
    '1':'5'
}

def getFrequencyFromTradeType(ktype):
    return FrequencyDic[ktype]

def getFrequencyList():
    return FrequencyDic.keys()


def getMinsDiffFromFreq(freq):
    return FrequencyMinDic[freq]

def getSubFreq(freq):
    if SubFreqDic.__contains__(freq):
        return SubFreqDic[freq]
    return None

def getFreqDesc(freq):
    return FrequencyDescDic[freq]

def getDayFreqList():
    return ['D','W','M']

def getMinFreqList():
    return ['1','5','15','30','60']

def getStorageFreqList():
    return ['D','1','5','15','30','60']

def getSuperFreq(freq):
    return SuperFreqDic[freq]



__author__ = 'shenli'
import UtilTools.DateUtils as DateUtils
import ChangLunLogic.SimplifyLogic as SimplifyLogic
import ChangLunLogic.PartingLogic as PartingLogic
import ChangLunLogic.PenLogic as PenLogic
import ChangLunLogic.SegmentLogic as SegmentLogic
import ChangLunLogic.CentreLogic as CentreLogic
import ChangLunLogic.BuyPointLogic as BuyPointLogic
import UtilTools.FrequencyUtils as FreqUtils
from ChangLunLogic.Model import *
import pandas as pd

class ChangLun:
    def __init__(self, stockDf, baseFreq,rootFreq):
        self.baseFreq = baseFreq
        self.rootFreq = rootFreq
        self.buyPoints = {}
        self.histData = self.convertDfToHistData(stockDf)
        self.lastDate = self.histData[-2]['date']
        self.stockId = stockDf['stock_id'].to_list()[0]
        self.tradeDate = stockDf['date'].to_list()[-1]

    def convertDfToHistData(self, stockDf):
        if isinstance(stockDf,pd.DataFrame):
            return stockDf.to_dict(orient ='records')
        return stockDf

    def isPenBuyPoint(self):
        if self.penList is None or len(self.penList) == 0:
            return False
        lastPen = self.penList[-1]
        if lastPen['type'] == PenType.down and lastPen['endOrginIndex'] == len(self.histData) - 2:
            return True
        return False

    def run(self, backTest = False):
        self.simplifyData = SimplifyLogic.simplifyHistData(self.histData)
        self.partingList = PartingLogic.getPartingList(self.simplifyData)
        if backTest:
            if not self.isLastPartingDown():
                return False
        self.penList = PenLogic.createPenList(self.partingList)
        self.baseSegmentList = SegmentLogic.createSegmentList(self.penList)
        self._initSegmentDic()
        self._initCentreDic()
        self._initPenCentre()

        return True
        #self.buyPoint = BuyPointLogic.getThirtyLevelBuyPoint(self.segmentDic,self.endDateStr)
        #self.buyPointFive = BuyPointLogic.getThirtyLevelBuyPoint(self.segmentDic,self.endDateStr,'5')
        #self._initBuyPoints()

    def isPenChange(self):
        if self.penList is None:
            return False
        lastPen = self.penList[-1]
        if lastPen['endSimplifyIndex'] == len(self.simplifyData) - 3:
            #print 'pen change'
            return True
        return False

    def getBuyPointsCount(self):
        count = 0
        for freq in self.segmentDic.keys():
            if self.segmentDic[freq] is not None and len(self.segmentDic[freq]) > 0:
                lastSegmentEndDateStr = DateUtils.convertTimestampToStr(self.segmentDic[freq][-1]['endDate'])
                if lastSegmentEndDateStr[0:10] == self.endDateStr[0:10] and self.segmentDic[freq][-1]['type'] == PenType.down:
                    count += 1
        return count

    def getLastPen(self):
        return self.penList[-1]

    def isSegmentChange(self,freq='1'):
        if self.segmentDic is None or not self.segmentDic.has_key(freq) or self.segmentDic[freq] is None:
            return False
        lastSegment = self.segmentDic[freq][-1]
        if lastSegment['endSimplifyIndex'] == len(self.simplifyData) - 3:
            return True
        return False

    def getLastSegment(self,freq='1'):
        return self.segmentDic[freq][-1]

    def _initSegmentDic(self):
        self.segmentDic = {}
        subFreq = FreqUtils.getSubFreq(self.baseFreq)
        self.segmentDic[subFreq] = self.penList
        self.segmentDic[self.baseFreq] = self.baseSegmentList
        currentFreq = self.baseFreq
        superFreq = FreqUtils.getSuperFreq(self.baseFreq)
        while superFreq is not None and superFreq != FreqUtils.getSuperFreq(self.rootFreq):
            currentSegment = self.segmentDic[currentFreq]
            superSegment = SegmentLogic.createSegmentList(currentSegment)
            self.segmentDic[superFreq] = superSegment
            currentFreq = superFreq
            superFreq = FreqUtils.getSuperFreq(currentFreq)

    def _initCentreDic(self):
        self.centreDic = {}
        currentFreq = self.baseFreq
        superFreq = FreqUtils.getSuperFreq(currentFreq)
        while superFreq is not None and self.segmentDic.__contains__(superFreq):
            currentSegment = self.segmentDic[currentFreq]
            superSegment = self.segmentDic[superFreq]
            centreList = CentreLogic.createCentreList(superSegment,currentSegment)
            self.centreDic[currentFreq] = centreList
            currentFreq = superFreq
            superFreq = FreqUtils.getSuperFreq(currentFreq)

    def _initPenCentre(self):
        baseSegment = self.segmentDic[self.baseFreq]
        self.penCentre = CentreLogic.createCentreList(baseSegment, self.penList)
        self.centreDic['pen'] = self.penCentre


    def getSegment(self,freq):
        if self.segmentDic.__contains__(freq):
            return self.segmentDic[freq]
        return None

    def getCentre(self,freq):
        if self.centreDic.__contains__(freq):
            return self.centreDic[freq]
        return None

    def isLastPartingDown(self):
        if self.partingList is None or len(self.partingList) == 0:
            return False
        lastParting = self.partingList[-1]
        if lastParting['type'] == PartingType.down and lastParting['simplifyIndex'] == len(self.simplifyData) -2:
            return True
        return False

    def genPenFeatures(self):
        feature = {}
        lastPen = self.penList[-1]
        lastPenType = lastPen['type']
        penEndDate = lastPen['endDate']
        penStartValue = lastPen['startValue']
        penEndValue = lastPen['endValue']
        isLastPenPoint = penEndDate == self.lastDate
        isPenBuyPoint = (lastPenType == PenType.down and isLastPenPoint)
        isPenSellPoint = (lastPenType == PenType.up and isLastPenPoint)
        penChgRatio = (penEndValue - penStartValue) / penStartValue * 100
        penHistNum = lastPen['endOrginIndex'] - lastPen['startOrginIndex'] + 1
        penChgVelocity = penChgRatio / penHistNum
        penPartingNum = lastPen['endPartingIndex'] - lastPen['startPartingIndex'] + 1
        penPartingList = self.partingList[lastPen['startPartingIndex']:lastPen['endPartingIndex']]
        penUpPartingNum = 0
        penDownPartingNum = 0
        for parting in penPartingList:
            type = parting['type']
            if type == PartingType.down:
                penDownPartingNum += 1
            if type == PartingType.up:
                penUpPartingNum += 1
        feature['is_last_pen_point'] = isLastPenPoint
        feature['is_pen_buy_point'] = isPenBuyPoint
        feature['is_pen_sell_point'] = isPenSellPoint
        feature['pen_chg_ratio'] = penChgRatio
        feature['pen_chg_velocity'] = penChgVelocity
        feature['pen_parting_num'] = penPartingNum
        feature['pen_hist_num'] = penHistNum
        feature['pen_up_parting_num'] = penUpPartingNum
        feature['pen_down_parting_num'] = penDownPartingNum
        feature['pen_end_date'] = penEndDate
        return feature

    def genPartingFeature(self):
        lastParting = self.partingList[-1]
        feature = {}
        feature['parting_type'] = lastParting['type']
        feature['parting_right_mid_high_chg'] = (lastParting['rightHigh'] - lastParting['midHigh']) / lastParting['midHigh'] * 100
        feature['parting_right_mid_low_chg'] = (lastParting['rightLow'] - lastParting['midLow']) / lastParting['midLow'] * 100
        feature['parting_right_left_high_chg'] = (lastParting['rightHigh'] - lastParting['leftHigh']) / lastParting['midHigh'] * 100
        feature['parting_right_left_low_chg'] = (lastParting['rightLow'] - lastParting['leftLow']) / lastParting['midLow'] * 100
        feature['parting_mid_left_high_chg'] = (lastParting['midHigh'] - lastParting['leftHigh']) / lastParting['midHigh'] * 100
        feature['parting_mid_left_low_chg'] = (lastParting['midLow'] - lastParting['leftLow']) / lastParting['midLow'] * 100
        high = max([lastParting['leftHigh'],lastParting['midHigh'],lastParting['rightHigh']])
        low = min([lastParting['leftLow'],lastParting['midLow'],lastParting['rightLow']])
        feature['parting_chg_ratio'] = (high - low) / low * 100
        feature['parting_left_chg'] = (lastParting['leftHigh'] - lastParting['leftLow']) / lastParting['leftLow'] * 100
        feature['parting_mid_chg'] = (lastParting['midHigh'] - lastParting['midLow']) / lastParting['midLow'] * 100
        feature['parting_right_chg'] = (lastParting['rightHigh'] - lastParting['rightLow']) / lastParting['rightLow'] * 100
        feature['parting_right_low_mid_high_chg'] = (lastParting['rightLow'] - lastParting['midHigh']) / lastParting['rightLow'] * 100
        feature['parting_left_low_mid_high_chg'] = (lastParting['leftLow'] - lastParting['midHigh']) / lastParting['midHigh'] * 100
        return feature

    def genSegmentFeature(self,freq):
        segmentList = self.segmentDic[freq]
        if segmentList is None or len(segmentList) == 0:
            return {}
        lastSegment = segmentList[-1]
        feature = {}
        lastSegmentType = lastSegment['type']
        endDate = lastSegment['endDate']
        startValue = lastSegment['startValue']
        endValue = lastSegment['endValue']
        isLastSegmentPoint = endDate == self.lastDate
        isSegmentBuyPoint = (lastSegmentType == PenType.down and isLastSegmentPoint)
        isPenSellPoint = (lastSegmentType == PenType.up and isLastSegmentPoint)
        segmentChgRatio = (endValue - startValue) / startValue * 100
        segmentHistNum = lastSegment['endOrginIndex'] - lastSegment['startOrginIndex'] + 1
        segmentChgVelocity = segmentChgRatio / segmentHistNum
        segmentPenNum = lastSegment['endPenIndex'] - lastSegment['startPenIndex'] + 1
        freqLower = str(freq).lower()
        feature['%s_is_last_segment_point' % (freqLower)] = isLastSegmentPoint
        feature['%s_is_segment_buy_point' % (freqLower)] = isSegmentBuyPoint
        feature['%s_is_segment_sell_point' % (freqLower)] = isPenSellPoint
        feature['%s_segment_chg_ratio' % (freqLower)] = segmentChgRatio
        feature['%s_segment_chg_velocity' % (freqLower)] = segmentChgVelocity
        feature['%s_segment_pen_num' % (freqLower)] = segmentPenNum
        feature['%s_segment_centre_num' % (freqLower)] = lastSegment['centerNum']
        feature['%s_segment_bar_num' % (freqLower)] = segmentHistNum
        return feature

    def genCentreFeature(self,freq):
        if not self.centreDic.__contains__(freq):
            return {}
        centreList = self.centreDic[freq]
        if centreList is None or len(centreList) == 0:
            return {}
        feature = {}
        lastCentre = centreList[-1]
        low = lastCentre['low']
        high = lastCentre['high']
        extremeLow = lastCentre['extremeLow']
        extremeHigh = lastCentre['extremeHigh']
        freqLower = str(freq).lower()
        feature['%s_centre_chg_ratio' % (freqLower)] = (high - low) / low * 100
        feature['%s_centre_extreme_chg_ratio' % (freqLower)] = (extremeHigh - extremeLow) / extremeLow * 100
        feature['%s_centre_hist_num' % (freqLower)] = lastCentre['endOrginIndex'] - lastCentre['startOrginIndex'] + 1
        feature['%s_centre_pen_num' % (freqLower)] = lastCentre['endPenIndex'] - lastCentre['startPenIndex'] + 1
        lastPen = self.penList[-1]
        lastPenValue = lastPen['endValue']
        feature['%s_pen_to_centre_upper' % (freqLower)] = (lastPenValue - high) / high * 100
        feature['%s_pen_to_centre_lower' % (freqLower)] = (lastPenValue - low) / low * 100
        feature['%s_pen_to_centre_extreme_upper' % (freqLower)] = (lastPenValue - extremeHigh) / extremeHigh * 100
        feature['%s_pen_to_centre_extreme_lower' % (freqLower)] = (lastPenValue - extremeLow) / extremeLow * 100
        return feature

    def genFeature(self):
        feature = {}
        if self.penList is None or len(self.penList) == 0:
            return {}
        feature['stock_id'] = self.stockId
        feature['trade_date'] = self.tradeDate
        feature['buy_date'] = self.penList[-1]['endDate']
        feature['buy_value'] = self.penList[-1]['endValue']
        partingFeature = self.genPartingFeature()
        penFeature = self.genPenFeatures()
        segmentFeatureD = self.genSegmentFeature('D')
        #segmentFeatureW = self.genSegmentFeature('W')
        centreFeaturePen = self.genCentreFeature('pen')
        centreFeatureD = self.genCentreFeature('D')
        barFeature = self.genBarFeature()
        feature.update(partingFeature)
        feature.update(penFeature)
        feature.update(segmentFeatureD)
        #feature.update(segmentFeatureW)
        feature.update(centreFeatureD)
        feature.update(centreFeaturePen)
        feature.update(barFeature)
        return feature

    def genBarFeature(self):
        feature = {}
        bar = self.histData[-1]
        beforeBar = self.histData[-2]
        barOpenChg = (bar['open'] - beforeBar['close']) / beforeBar['close'] * 100
        barMaxChg = (bar['high'] - bar['low']) / bar['low'] * 100
        barMaxToPartingChg = (bar['high'] - beforeBar['low']) / beforeBar['low'] * 100
        barHighToCloseChg = (bar['high'] - bar['close']) / bar['close'] * 100
        barCloseToLowChg = (bar['close'] - bar['low']) / bar['low'] * 100
        barCloseToOpenChg = (bar['close'] - bar['open']) / bar['open'] * 100
        avgPrice = (bar['high'] + bar['low']) / 2
        barCloseToMinChg = (bar['close'] - avgPrice) / avgPrice * 100
        feature['bar_open_chg'] = barOpenChg
        feature['bar_max_chg'] = barMaxChg
        feature['bar_max_to_parting_chg'] = barMaxToPartingChg
        feature['bar_high_to_close_chg'] = barHighToCloseChg
        feature['bar_close_to_low_chg'] = barCloseToLowChg
        feature['bar_close_to_open_chg'] = barCloseToOpenChg
        feature['bar_close_to_min_chg'] = barCloseToMinChg
        return feature

'''
    def _initBuyPoints(self):
        freq = FreqUtils.getSubFreq(self.rootFreq)
        while freq != self.baseFreq:
            self._initBuyPointsFreq(freq)
            freq = FreqUtils.getSubFreq(freq)

    def _initBuyPointsFreq(self,freq):
        self.buyPoints[freq] = {}
        self._initFisrtBuy(freq)
        self._initSecondBuy(freq)
        self._initThirdBuy(freq)

    def _initFisrtBuy(self,freq):
        superFreq = FreqUtils.getSuperFreq(freq)
        if not self.segmentDic.has_key(superFreq):
            return
        superSegment = self.segmentDic[superFreq]
        if superSegment is None or len(superSegment) == 0:
            return
        lastTrend = superSegment[-1]
        if lastTrend['type'] == PenType.up:
            return
        if self._isSegmentsDownDeparture(freq,lastTrend):
            buyPoint = {}
            buyPoint['type'] = BuyPointsType.firstBuy
            buyPoint['date'] = lastTrend['endDate']
            buyPoint['value'] = lastTrend['endValue']
            buyPoint['orginIndex'] = lastTrend['endOrginIndex']
            buyPoint['segmentIndex'] = lastTrend['endPenIndex']
            buyPoint['freq'] = freq
            buyDic = self.buyPoints[freq]
            buyDic[BuyPointsType.firstBuy] = buyPoint

    def _initSecondBuy(self,freq):
        if not self.buyPoints.has_key(freq):
            return
        buyDic = self.buyPoints[freq]
        if buyDic is None or not buyDic.has_key(BuyPointsType.firstBuy):
            return
        firstBuy = buyDic[BuyPointsType.firstBuy]
        firstBuySegmentIndex = firstBuy['segmentIndex']
        segmentList = self.getSegment(freq)
        if firstBuySegmentIndex + 2 > len(segmentList) - 1:
            return None
        secondBuySegment = segmentList[firstBuySegmentIndex + 2]
        subFreq = FreqUtils.getSubFreq(freq)
        if self._isSegmentsDownDeparture(subFreq,secondBuySegment):
            lastCentre = self.getCentre(freq)[-1]
            buyType = BuyPointsType.secondBuy
            if secondBuySegment['endValue'] > lastCentre['high']:
                buyType = BuyPointsType.secondThirdBuy
            buyPoint = {}
            buyPoint['type'] = buyType
            buyPoint['date'] = secondBuySegment['endDate']
            buyPoint['value'] = secondBuySegment['endValue']
            buyPoint['orginIndex'] = secondBuySegment['endOrginIndex']
            buyPoint['segmentIndex'] = secondBuySegment['index']
            buyPoint['freq'] = freq
            buyDic[buyType] = buyPoint

    def _initThirdBuy(self,freq):
        superFreq = FreqUtils.getSuperFreq(freq)
        if not self.segmentDic.has_key(superFreq):
            return
        superSegment = self.segmentDic[superFreq]
        if superSegment is None or len(superSegment) == 0:
            return
        lastTrend = superSegment[-1]
        if lastTrend['type'] == PenType.down:
            return
        segmentList = self.getSegment(freq)
        thirdSegmentIndex = lastTrend['endPenIndex'] + 1
        if thirdSegmentIndex > len(segmentList) - 1:
            return
        thirdSegment = segmentList[thirdSegmentIndex]
        centreList = self.getCentre(freq)
        if centreList is None or len(centreList) == 0:
            return
        lastCentre = centreList[-1]
        if thirdSegment['endValue'] < lastCentre['high']:
            return
        subFreq = FreqUtils.getSubFreq(freq)
        if self._isSegmentsDownDeparture(subFreq,thirdSegment):
            buyPoint = {}
            buyPoint['type'] = BuyPointsType.thirdBuy
            buyPoint['date'] = thirdSegment['endDate']
            buyPoint['value'] = thirdSegment['endValue']
            buyPoint['orginIndex'] = thirdSegment['endOrginIndex']
            buyPoint['segmentIndex'] = thirdSegment['endPenIndex']
            buyPoint['freq'] = freq
            buyDic = self.buyPoints[freq]
            buyDic[BuyPointsType.thirdBuy] = buyPoint


    def _isSegmentsDownDeparture(self,freq,superTrend):
        if not self.centreDic.has_key(freq):
            return False
        centreList = self.getCentre(freq)
        if centreList is None or len(centreList) == 0:
            return False
        lastCentre = None
        for i in range(0,len(centreList))[::-1]:
            if centreList[i]['segmentIndex'] == superTrend['index'] and centreList[i]['endPenIndex'] < superTrend['endPenIndex']:
                lastCentre = centreList[i]
                break
        if lastCentre is None:
            return False
        segmentList = self.getSegment(freq)
        if lastCentre['endPenIndex'] + 1 >= len(segmentList):
            return False
        beforeSegment = segmentList[lastCentre['startPenIndex'] - 1]
        afterSegment = segmentList[lastCentre['endPenIndex'] + 1]
        leftArea = self.macdLogic.getMacdArea(beforeSegment['startDate'],beforeSegment['endDate'],True,False)
        rightArea = self.macdLogic.getMacdArea(afterSegment['startDate'],afterSegment['endDate'],False,False)
        leftSignalDiff = self.macdLogic.getExtremeMacdSignalDiff(beforeSegment['startDate'],beforeSegment['endDate'])
        rightSignalDiff = self.macdLogic.getExtremeMacdSignalDiff(afterSegment['startDate'],afterSegment['endDate'])
        currentIsDeparture = (rightArea < leftArea) and (rightSignalDiff < leftSignalDiff)
        if currentIsDeparture:
            subFreq = FreqUtils.getSubFreq(freq)
            if not self.segmentDic.has_key(subFreq):
                return True
            else:
                return self._isSegmentsDownDeparture(subFreq,segmentList[-1])
        else:
            return False

    def getBuyPoints(self,freq):
        if not self.buyPoints.has_key(freq):
            return None
        return self.buyPoints[freq]

    def isReadyToBuy(self,freq):
        buyPoints = self.getBuyPoints(freq)
        if buyPoints is None:
            return False
        if buyPoints['orginIndex'] == len(self.histData) - 2:
            return True
        else:
            return False

    def getLastBuyPoint(self,freq):
        return self.getBuyPoints(freq)
'''





from UtilTools.impalaUtils import getDfFromSql
from ChangLunLogic.changLunBackTest import getHistTradeFeatures
from ChangLunLogic.ChangLun import ChangLun
from Plot.ChanLunPlot import ChanLunPlot
import pandas as pd
from UtilTools.mysqlClient import saveDf

def runForStock(stockId):
    sql = "SELECT stock_id, trade_date as `date`, open,close,low,high from dwd_ts_stock_daily where stock_id = '%s' order by trade_date asc" % (stockId)
    df = getDfFromSql('dwd',sql)
    features = getHistTradeFeatures(df)
    print(len(features))
    print(features)
    model = ChangLun(df,'D','W')
    model.run()
    if model.isPenBuyPoint():
        plot = ChanLunPlot(model)
        plot.setFeatures(features)
        plot.plot()
    if features is not None and len(features) > 0:
        featureDf = pd.DataFrame(features)
        saveDf(featureDf,'dwd','dwd_changlun_features',isAppend=True,index=['stock_id','trade_date'])

def run():
    sql = "SELECT distinct stock_id from dwd_ts_stock_daily"
    df = getDfFromSql('dwd',sql)
    stockList = df['stock_id'].to_list()
    for stock in stockList:
        print(stock)
        try:
            runForStock(stock)
        except:
            print('error... skip')

if __name__ == '__main__':
    #run()
    runForStock('000792')




def runProfile(functionName):
    import cProfile
    cProfile.run(functionName + "()", "result")
    import pstats
    p = pstats.Stats("result")
    p.strip_dirs().sort_stats("cumulative").print_stats()


import ChangLunLogic.SimplifyLogic as SimplifyLogic
import ChangLunLogic.PartingLogic as PartingLogic
import ChangLunLogic.PenLogic as PenLogic
from ChangLunLogic.Model import *

def isPenChange(penList,histData):
    beforePen = penList[0]
    lastPen = penList[-1]
    startOrginIndex = beforePen['startOrginIndex']
    tempData = histData[startOrginIndex:]
    simpleData = SimplifyLogic.simplifyHistData(tempData)
    lastPartingType = PartingLogic._getPartingType(simpleData[-3:])
    if lastPartingType == PartingType.none:
        return False
    partingList = PartingLogic.getPartingList(simpleData)
    lastPenParting = None
    for parting in partingList:
        if parting['date'] == lastPen['endDate']:
            lastPenParting = parting
            break
    if lastPenParting is None:
        return False
    if PenLogic._isPenExist(lastPenParting,partingList[-1]) or PenLogic._isSameTypeAndMoreHigherOrLower(lastPenParting,partingList[-1]):
        return True
    else:
        return False




from ChangLunLogic.Model import *

def getPartingList(simplifyData):
    if simplifyData is None or len(simplifyData) < 3:
        return None
    partingList = []
    for i in range(0,len(simplifyData)-2):
        appenParting(simplifyData,i,partingList)
    return partingList

def appenParting(simplifyData,i,partingList):
    window = 3
    partingType = _getPartingType(simplifyData[i:i + window])
    if partingType == PartingType.none:
        return
    orginIndex = simplifyData[i + 1]['orginIndex']
    date = simplifyData[i + 1]['date']
    if partingType == PartingType.down:
        value = simplifyData[i + 1]['low']
    else:
        value = simplifyData[i + 1]['high']
    extremePoints = _getPartingExtremePoints(simplifyData[i:i + window])
    parting = _createParting(i + 1, orginIndex, date, value, partingType, extremePoints)
    partingList.append(parting)
    partingList[-1]['index'] = len(partingList) - 1

def _getPartingExtremePoints(windowData):
    '''
    minValue = min(windowData,key = lambda x:x['low'])
    maxValue = max(windowData,key = lambda x:x['high'])
    return (minValue['low'],maxValue['high'])
    '''

    leftHigh = windowData[0]['high']
    leftLow = windowData[0]['low']
    midHigh = windowData[1]['high']
    midLow = windowData[1]['low']
    rightHigh = windowData[2]['high']
    rightLow = windowData[2]['low']
    return (leftLow,leftHigh,midLow,midHigh,rightLow,rightHigh)


def _createParting(simplifyIndex,orginIndex,date,value,type,extremePoints):
    parting = {}
    parting['orginIndex'] = orginIndex
    parting['simplifyIndex'] = simplifyIndex
    parting['date'] = date
    parting['value'] = value
    parting['type'] = type
    parting['leftLow'] = extremePoints[0]
    parting['leftHigh'] = extremePoints[1]
    parting['midLow'] = extremePoints[2]
    parting['midHigh'] = extremePoints[3]
    parting['rightLow'] = extremePoints[4]
    parting['rightHigh'] = extremePoints[5]
    return parting

def _getPartingType(windowData):
    assert len(windowData) == 3
    if windowData[1]['high'] > windowData[0]['high'] and windowData[1]['high'] > windowData[2]['high']:
        return PartingType.up
    if windowData[1]['low'] < windowData[0]['low'] and windowData[1]['low'] < windowData[2]['low']:
        return PartingType.down
    return PartingType.none



from influxdb import InfluxDBClient
import QUANTAXIS as QA
import UtilTools.DateUtils as DateUtils
import UtilTools.ProfileUtils as ProfileUtils
from datetime import datetime
from datetime import timedelta

host='192.168.1.153'
port=8086
user = 'admin'
password = 'admin'
dbname = 'stock'
client = InfluxDBClient(host, port, user, password, dbname)

def getTableName(stockId,freq):
    return "stock_%s_%s" % (stockId,freq)

def deleteTable(stockId,freq):
    tableName = getTableName(stockId,freq)
    client.drop_measurement(tableName)

def insertStockData(stockId,stockData,freq):
    tableName = getTableName(stockId,freq)
    points = []
    for data in stockData:
        point = {"measurement": tableName}
        point["tags"] = {"type": data['type']}
        point["time"] = DateUtils.convertTimestampToStr(data['date'])
        point['fields'] = {
            'date': data['date'].timestamp(),
            'open': round(data['open'],2),
            'close':round(data['close'],2),
            'high':round(data['high'],2),
            'low':round(data['low'],2),
            'volume':round(data['volume']),
            'amount':data['amount'],
            'code': stockId
        }
        points.append(point)
    client.write_points(points)

def initStockData(stockId,startDate,endDate,freq):
    stockMin = QA.QA_fetch_stock_min_adv(stockId, startDate, endDate, frequence=freq).to_qfq()
    histData = stockMin.data
    histData['date'] = histData.index.levels[0]
    stockData = histData.to_dict('records')
    points = []
    tableName = getTableName(stockId,freq)
    for data in stockData:
        point = {"measurement": tableName}
        point["tags"] = {"type": data['type']}
        point["time"] = DateUtils.convertTimestampToStr(data['date'])
        point['fields'] = {
            'date': data['date'].timestamp()  - 28800,
            'open': round(data['open'],2),
            'close':round(data['close'],2),
            'high':round(data['high'],2),
            'low':round(data['low'],2),
            'volume':round(data['volume']),
            'amount':data['amount']

        }
        points.append(point)
    client.write_points(points)

def getStockData(stockId,freq):
    tableName = getTableName(stockId,freq)
    rs = client.query("select * from %s" % (tableName))
    result = list(rs.get_points(measurement=tableName))
    for data in result:
        data['date'] = datetime.fromtimestamp(data['date'])
    return result

def getStockDataFromDate(stockId,freq,startDate,endDate):
    tableName = getTableName(stockId,freq)
    rs = client.query("select * from %s where time >= '%s' and time <= '%s'" % (tableName,startDate,endDate))
    result = list(rs.get_points(measurement=tableName))
    for data in result:
        data['date'] = datetime.fromtimestamp(data['date'])
    return result

def getLastDatetime(stockId,freq):
    tableName = getTableName(stockId,freq)
    rs = client.query("select max(date) as date from %s" % (tableName))
    result = list(rs.get_points(measurement=tableName))
    date = datetime.fromtimestamp(result[0]['date'])
    print(date)
    return date

def deleteData(stockId,freq,startDate):
    tableName = getTableName(stockId,freq)
    rs = client.query("delete from %s where time > '%s'" % (tableName,startDate))

def runTest():
    stockId = '000001'
    startDate = '2018-03-26'
    endDate = '2019-03-26'
    freq = '5m'
    #saveStockData(stockId,startDate,endDate,freq)
    #dataList = getStockData(stockId,freq)
    getLastDatetime(stockId,freq)

if __name__ == '__main__':
    deleteData('000001','5m')
